@@ -23,6 +23,8 @@
 #include <geos/io/WKBWriter.h>
 #include <geos/io/GeoJSONReader.h>
 #include <geos/io/GeoJSONWriter.h>
+#include <geos/operation/buffer/BufferParameters.h>
+#include <geos/operation/cluster/Clusters.h>
 #include <geos/util/Interrupt.h>
 
 #include <stdexcept>
@@ -32,10 +34,16 @@
 #pragma warning(disable : 4099)
 #endif
 
-// Some extra magic to make type declarations in geos_c.h work - for cross-checking of types in header.
+// Some extra magic to make type declarations in geos_c.h work -
+// for cross-checking of types in header.
+// NOTE: the below defines or struct definition must be kept in exact
+// sync between geos_c.cpp and geos_ts_c.cpp to avoid C++ One Definition Rule
+// violations.
 #define GEOSGeometry geos::geom::Geometry
 #define GEOSPreparedGeometry geos::geom::prep::PreparedGeometry
+#define GEOSClusterInfo geos::operation::cluster::Clusters
 #define GEOSCoordSequence geos::geom::CoordinateSequence
+#define GEOSBufferParams geos::operation::buffer::BufferParameters
 #define GEOSSTRtree geos::index::strtree::TemplateSTRtree<void*>
 #define GEOSWKTReader geos::io::WKTReader
 #define GEOSWKTWriter geos::io::WKTWriter
@@ -43,8 +51,20 @@
 #define GEOSWKBWriter geos::io::WKBWriter
 #define GEOSGeoJSONReader geos::io::GeoJSONReader
 #define GEOSGeoJSONWriter geos::io::GeoJSONWriter
-typedef struct GEOSBufParams_t GEOSBufferParams;
-typedef struct GEOSMakeValidParams_t GEOSMakeValidParams;
+
+// Implementation struct for the GEOSCoverageCleanParams object
+typedef struct {
+    double snappingDistance;
+    int    overlapMergeStrategy;
+    double gapMaximumWidth;
+} GEOSCoverageCleanParams;
+
+// Implementation struct for the GEOSMakeValidParams object
+typedef struct {
+    int method;
+    int keepCollapsed;
+} GEOSMakeValidParams;
+
 
 #include "geos_c.h"
 
@@ -136,7 +156,7 @@ extern "C" {
 
     /****************************************************************
     ** relate()-related functions
-    ** return 0 = false, 1 = true, 2 = error occured
+    ** return 0 = false, 1 = true, 2 = error occurred
     **
     */
     char
@@ -203,15 +223,15 @@ extern "C" {
 //------------------------------------------------------------------
 
     char
-    GEOSRelatePattern(const Geometry* g1, const Geometry* g2, const char* pat)
+    GEOSRelatePattern(const Geometry* g1, const Geometry* g2, const char* imPattern)
     {
-        return GEOSRelatePattern_r(handle, g1, g2, pat);
+        return GEOSRelatePattern_r(handle, g1, g2, imPattern);
     }
 
     char
-    GEOSRelatePatternMatch(const char* mat, const char* pat)
+    GEOSRelatePatternMatch(const char* intMatrix, const char* imPattern)
     {
-        return GEOSRelatePatternMatch_r(handle, mat, pat);
+        return GEOSRelatePatternMatch_r(handle, intMatrix, imPattern);
     }
 
     char*
@@ -267,6 +287,12 @@ extern "C" {
         return GEOSEqualsExact_r(handle, g1, g2, tolerance);
     }
 
+    char
+    GEOSEqualsIdentical(const Geometry* g1, const Geometry* g2)
+    {
+        return GEOSEqualsIdentical_r(handle, g1, g2);
+    }
+
     int
     GEOSDistance(const Geometry* g1, const Geometry* g2, double* dist)
     {
@@ -327,6 +353,61 @@ extern "C" {
         return GEOSNearestPoints_r(handle, g1, g2);
     }
 
+    GEOSClusterInfo*
+    GEOSClusterDBSCAN(const GEOSGeometry* g, double eps, unsigned minPoints)
+    {
+        return GEOSClusterDBSCAN_r(handle, g, eps, minPoints);
+    }
+
+    GEOSClusterInfo*
+    GEOSClusterGeometryDistance(const GEOSGeometry* g, double d)
+    {
+        return GEOSClusterGeometryDistance_r(handle, g, d);
+    }
+
+    GEOSClusterInfo*
+    GEOSClusterGeometryIntersects(const GEOSGeometry* g)
+    {
+        return GEOSClusterGeometryIntersects_r(handle, g);
+    }
+
+    GEOSClusterInfo*
+    GEOSClusterEnvelopeDistance(const GEOSGeometry* g, double d)
+    {
+        return GEOSClusterEnvelopeDistance_r(handle, g, d);
+    }
+
+    GEOSClusterInfo*
+    GEOSClusterEnvelopeIntersects(const GEOSGeometry* g)
+    {
+        return GEOSClusterEnvelopeIntersects_r(handle, g);
+    }
+
+    std::size_t GEOSClusterInfo_getNumClusters(const GEOSClusterInfo* clusters)
+    {
+        return GEOSClusterInfo_getNumClusters_r(handle, clusters);
+    }
+
+    std::size_t GEOSClusterInfo_getClusterSize(const GEOSClusterInfo* clusters, size_t i)
+    {
+        return GEOSClusterInfo_getClusterSize_r(handle, clusters, i);
+    }
+
+    const std::size_t* GEOSClusterInfo_getInputsForClusterN(const GEOSClusterInfo* clusters, size_t i)
+    {
+        return GEOSClusterInfo_getInputsForClusterN_r(handle, clusters, i);
+    }
+
+    std::size_t* GEOSClusterInfo_getClustersForInputs(const GEOSClusterInfo* clusters)
+    {
+        return GEOSClusterInfo_getClustersForInputs_r(handle, clusters);
+    }
+
+    void GEOSClusterInfo_destroy(GEOSClusterInfo* info)
+    {
+        GEOSClusterInfo_destroy_r(handle, info);
+    }
+
     Geometry*
     GEOSGeomFromWKT(const char* wkt)
     {
@@ -378,6 +459,12 @@ extern "C" {
         return GEOSisSimple_r(handle, g);
     }
 
+    char
+    GEOSisSimpleDetail(const Geometry* g, int returnAllPoints, Geometry** result)
+    {
+        return GEOSisSimpleDetail_r(handle, g, returnAllPoints, result);
+    }
+
     char
     GEOSisRing(const Geometry* g)
     {
@@ -477,6 +564,15 @@ extern "C" {
         return GEOSConcaveHull_r(handle, g, ratio, allowHoles);
     }
 
+    Geometry*
+    GEOSConcaveHullByLength(const Geometry* g,
+                    double length,
+                    unsigned int allowHoles)
+
+    {
+        return GEOSConcaveHullByLength_r(handle, g, length, allowHoles);
+    }
+
     Geometry*
     GEOSPolygonHullSimplify(const Geometry* g,
                             unsigned int isOuter,
@@ -600,6 +696,12 @@ extern "C" {
         return GEOSCoverageUnion_r(handle, g);
     }
 
+    Geometry*
+    GEOSDisjointSubsetUnion(const Geometry* g)
+    {
+        return GEOSDisjointSubsetUnion_r(handle, g);
+    }
+
     Geometry*
     GEOSNode(const Geometry* g)
     {
@@ -625,6 +727,12 @@ extern "C" {
         return GEOSClipByRect_r(handle, g, xmin, ymin, xmax, ymax);
     }
 
+    int
+    GEOSGridIntersectionFractions(const Geometry* g, double xmin, double ymin, double xmax, double ymax,
+                                  unsigned nx, unsigned ny, float* buf)
+    {
+        return GEOSGridIntersectionFractions_r(handle, g, xmin, ymin, xmax, ymax, nx, ny, buf);
+    }
 
     Geometry*
     GEOSGeom_transformXY(const GEOSGeometry* g, GEOSTransformXYCallback callback, void* userdata) {
@@ -632,6 +740,12 @@ extern "C" {
     }
 
 
+    Geometry*
+    GEOSGeom_transformXYZ(const GEOSGeometry* g, GEOSTransformXYZCallback callback, void* userdata) {
+        return GEOSGeom_transformXYZ_r(handle, g, callback, userdata);
+    }
+
+
 //-------------------------------------------------------------------
 // memory management functions
 //------------------------------------------------------------------
@@ -660,6 +774,12 @@ extern "C" {
         return GEOSNormalize_r(handle, g);
     }
 
+    int
+    GEOSOrientPolygons(Geometry* g, int exteriorCW)
+    {
+        return GEOSOrientPolygons_r(handle, g, exteriorCW);
+    }
+
     int
     GEOSGetNumInteriorRings(const Geometry* g)
     {
@@ -773,6 +893,16 @@ extern "C" {
         return GEOSGeomGetZ_r(handle, g1, z);
     }
 
+    /*
+     * For POINT
+     * returns 0 on exception, otherwise 1
+     */
+    int
+    GEOSGeomGetM(const Geometry* g1, double* m)
+    {
+        return GEOSGeomGetM_r(handle, g1, m);
+    }
+
     /*
      * Call only on polygon
      * Return a copy of the internal Geometry.
@@ -818,6 +948,12 @@ extern "C" {
         return GEOSGeom_createCollection_r(handle, type, geoms, ngeoms);
     }
 
+    Geometry**
+    GEOSGeom_releaseCollection(Geometry* collection, unsigned int * ngeoms)
+    {
+        return GEOSGeom_releaseCollection_r(handle, collection, ngeoms);
+    }
+
     Geometry*
     GEOSPolygonize(const Geometry* const* g, unsigned int ngeoms)
     {
@@ -891,6 +1027,60 @@ extern "C" {
         return GEOSMakeValidWithParams_r(handle, g, params);
     }
 
+    GEOSCoverageCleanParams*
+    GEOSCoverageCleanParams_create()
+    {
+        return GEOSCoverageCleanParams_create_r(handle);
+    }
+
+    void
+    GEOSCoverageCleanParams_destroy(
+        GEOSCoverageCleanParams* params)
+    {
+        return GEOSCoverageCleanParams_destroy_r(handle, params);
+    }
+
+    int
+    GEOSCoverageCleanParams_setSnappingDistance(
+        GEOSCoverageCleanParams* params, double snappingDistance)
+    {
+        return GEOSCoverageCleanParams_setSnappingDistance_r(
+            handle, params, snappingDistance);
+    }
+
+    int
+    GEOSCoverageCleanParams_setGapMaximumWidth(
+        GEOSCoverageCleanParams* params, double gapMaximumWidth)
+    {
+        return GEOSCoverageCleanParams_setGapMaximumWidth_r(
+            handle, params, gapMaximumWidth);
+    }
+
+    int
+    GEOSCoverageCleanParams_setOverlapMergeStrategy(
+        GEOSCoverageCleanParams* params, int overlapMergeStrategy)
+    {
+        return GEOSCoverageCleanParams_setOverlapMergeStrategy_r(
+            handle, params, overlapMergeStrategy);
+    }
+
+    GEOSGeometry *
+    GEOSCoverageCleanWithParams(
+        const GEOSGeometry* input,
+        const GEOSCoverageCleanParams* params)
+    {
+        return GEOSCoverageCleanWithParams_r(
+            handle, input, params);
+    }
+
+    GEOSGeometry *
+    GEOSCoverageClean(
+        const GEOSGeometry * input)
+    {
+        return GEOSCoverageClean_r(
+            handle, input);
+    }
+
     Geometry*
     GEOSRemoveRepeatedPoints(
         const Geometry* g,
@@ -911,6 +1101,12 @@ extern "C" {
         return GEOSLineMergeDirected_r(handle, g);
     }
 
+    Geometry*
+    GEOSLineSubstring(const Geometry* g, double start_fraction, double end_fraction)
+    {
+        return GEOSLineSubstring_r(handle, g, start_fraction, end_fraction);
+    }
+
     Geometry*
     GEOSReverse(const Geometry* g)
     {
@@ -947,6 +1143,12 @@ extern "C" {
         return GEOSHasZ_r(handle, g);
     }
 
+    char
+    GEOSHasM(const Geometry* g)
+    {
+        return GEOSHasM_r(handle, g);
+    }
+
     int
     GEOS_getWKBOutputDims()
     {
@@ -977,6 +1179,12 @@ extern "C" {
         return GEOSCoordSeq_create_r(handle, size, dims);
     }
 
+    CoordinateSequence*
+    GEOSCoordSeq_createWithDimensions(unsigned int size, int hasZ, int hasM)
+    {
+        return GEOSCoordSeq_createWithDimensions_r(handle, size, hasZ, hasM);
+    }
+
     CoordinateSequence*
     GEOSCoordSeq_copyFromBuffer(const double* buf, unsigned int size, int hasZ, int hasM)
     {
@@ -1001,6 +1209,18 @@ extern "C" {
         return GEOSCoordSeq_copyToArrays_r(handle, s, x, y, z, m);
     }
 
+    char
+    GEOSCoordSeq_hasZ(CoordinateSequence* s)
+    {
+        return GEOSCoordSeq_hasZ_r(handle, s);
+    }
+
+    char
+    GEOSCoordSeq_hasM(CoordinateSequence* s)
+    {
+        return GEOSCoordSeq_hasM_r(handle, s);
+    }
+
     int
     GEOSCoordSeq_setOrdinate(CoordinateSequence* s, unsigned int idx, unsigned int dim, double val)
     {
@@ -1010,19 +1230,25 @@ extern "C" {
     int
     GEOSCoordSeq_setX(CoordinateSequence* s, unsigned int idx, double val)
     {
-        return GEOSCoordSeq_setOrdinate(s, idx, 0, val);
+        return GEOSCoordSeq_setOrdinate(s, idx, CoordinateSequence::X, val);
     }
 
     int
     GEOSCoordSeq_setY(CoordinateSequence* s, unsigned int idx, double val)
     {
-        return GEOSCoordSeq_setOrdinate(s, idx, 1, val);
+        return GEOSCoordSeq_setOrdinate(s, idx, CoordinateSequence::Y, val);
     }
 
     int
     GEOSCoordSeq_setZ(CoordinateSequence* s, unsigned int idx, double val)
     {
-        return GEOSCoordSeq_setOrdinate(s, idx, 2, val);
+        return GEOSCoordSeq_setOrdinate(s, idx, CoordinateSequence::Z, val);
+    }
+
+    int
+    GEOSCoordSeq_setM(CoordinateSequence* s, unsigned int idx, double val)
+    {
+        return GEOSCoordSeq_setOrdinate(s, idx, CoordinateSequence::M, val);
     }
 
     int
@@ -1052,19 +1278,25 @@ extern "C" {
     int
     GEOSCoordSeq_getX(const CoordinateSequence* s, unsigned int idx, double* val)
     {
-        return GEOSCoordSeq_getOrdinate(s, idx, 0, val);
+        return GEOSCoordSeq_getOrdinate(s, idx, CoordinateSequence::X, val);
     }
 
     int
     GEOSCoordSeq_getY(const CoordinateSequence* s, unsigned int idx, double* val)
     {
-        return GEOSCoordSeq_getOrdinate(s, idx, 1, val);
+        return GEOSCoordSeq_getOrdinate(s, idx, CoordinateSequence::Y, val);
     }
 
     int
     GEOSCoordSeq_getZ(const CoordinateSequence* s, unsigned int idx, double* val)
     {
-        return GEOSCoordSeq_getOrdinate(s, idx, 2, val);
+        return GEOSCoordSeq_getOrdinate(s, idx, CoordinateSequence::Z, val);
+    }
+
+    int
+    GEOSCoordSeq_getM(const CoordinateSequence* s, unsigned int idx, double* val)
+    {
+        return GEOSCoordSeq_getOrdinate(s, idx, CoordinateSequence::M, val);
     }
 
     int
@@ -1139,6 +1371,24 @@ extern "C" {
         return GEOSGeom_createPolygon_r(handle, shell, holes, nholes);
     }
 
+    Geometry*
+    GEOSGeom_createCircularString(CoordinateSequence* cs)
+    {
+        return GEOSGeom_createCircularString_r(handle, cs);
+    }
+
+    Geometry*
+    GEOSGeom_createCompoundCurve(Geometry** curves, unsigned int ngeoms)
+    {
+        return GEOSGeom_createCompoundCurve_r(handle, curves, ngeoms);
+    }
+
+    Geometry*
+    GEOSGeom_createCurvePolygon(Geometry* shell, Geometry** holes, unsigned int nholes)
+    {
+        return GEOSGeom_createCurvePolygon_r(handle, shell, holes, nholes);
+    }
+
     Geometry*
     GEOSGeom_clone(const Geometry* g)
     {
@@ -1388,6 +1638,11 @@ extern "C" {
         GEOSWKBWriter_setIncludeSRID_r(handle, writer, newIncludeSRID);
     }
 
+    int
+    GEOS_printDouble(double d, unsigned int precision, char *result) {
+        return WKTWriter::writeTrimmedNumber(d, precision, result);
+    }
+
     /* GeoJSON Reader */
     GeoJSONReader*
     GEOSGeoJSONReader_create()
@@ -1426,6 +1681,18 @@ extern "C" {
         return GEOSGeoJSONWriter_writeGeometry_r(handle, writer, g, indent);
     }
 
+    void
+    GEOSGeoJSONWriter_setOutputDimension(GeoJSONWriter* writer, int dim)
+    {
+        GEOSGeoJSONWriter_setOutputDimension_r(handle, writer, dim);
+    }
+
+    int
+    GEOSGeoJSONWriter_getOutputDimension(GeoJSONWriter* writer)
+    {
+        return GEOSGeoJSONWriter_getOutputDimension_r(handle, writer);
+    }
+
 
 //-----------------------------------------------------------------
 // Prepared Geometry
@@ -1449,6 +1716,12 @@ extern "C" {
         return GEOSPreparedContains_r(handle, pg1, g2);
     }
 
+    char
+    GEOSPreparedContainsXY(const geos::geom::prep::PreparedGeometry* pg1, double x, double y)
+    {
+        return GEOSPreparedContainsXY_r(handle, pg1, x, y);
+    }
+
     char
     GEOSPreparedContainsProperly(const geos::geom::prep::PreparedGeometry* pg1, const Geometry* g2)
     {
@@ -1485,6 +1758,12 @@ extern "C" {
         return GEOSPreparedIntersects_r(handle, pg1, g2);
     }
 
+    char
+    GEOSPreparedIntersectsXY(const geos::geom::prep::PreparedGeometry* pg1, double x, double y)
+    {
+        return GEOSPreparedIntersectsXY_r(handle, pg1, x, y);
+    }
+
     char
     GEOSPreparedOverlaps(const geos::geom::prep::PreparedGeometry* pg1, const Geometry* g2)
     {
@@ -1503,6 +1782,18 @@ extern "C" {
         return GEOSPreparedWithin_r(handle, pg1, g2);
     }
 
+    char *
+    GEOSPreparedRelate(const geos::geom::prep::PreparedGeometry* pg1, const Geometry* g2)
+    {
+        return GEOSPreparedRelate_r(handle, pg1, g2);
+    }
+
+    char
+    GEOSPreparedRelatePattern(const geos::geom::prep::PreparedGeometry* pg1, const Geometry* g2, const char* imPattern)
+    {
+        return GEOSPreparedRelatePattern_r(handle, pg1, g2, imPattern);
+    }
+
     CoordinateSequence*
     GEOSPreparedNearestPoints(const geos::geom::prep::PreparedGeometry* g1, const Geometry* g2)
     {
@@ -1527,6 +1818,12 @@ extern "C" {
         return GEOSSTRtree_create_r(handle, nodeCapacity);
     }
 
+    int
+    GEOSSTRtree_build(GEOSSTRtree* tree)
+    {
+        return GEOSSTRtree_build_r(handle, tree);
+    }
+
     void
     GEOSSTRtree_insert(GEOSSTRtree* tree,
                        const geos::geom::Geometry* g,
@@ -1640,6 +1937,24 @@ extern "C" {
         return GEOSGeom_createEmptyPolygon_r(handle);
     }
 
+    geos::geom::Geometry*
+    GEOSGeom_createEmptyCircularString()
+    {
+        return GEOSGeom_createEmptyCircularString_r(handle);
+    }
+
+    geos::geom::Geometry*
+    GEOSGeom_createEmptyCompoundCurve()
+    {
+        return GEOSGeom_createEmptyCompoundCurve_r(handle);
+    }
+
+    geos::geom::Geometry*
+    GEOSGeom_createEmptyCurvePolygon()
+    {
+        return GEOSGeom_createEmptyCurvePolygon_r(handle);
+    }
+
     geos::geom::Geometry*
     GEOSGeom_createRectangle(double xmin, double ymin, double xmax,
                             double ymax)
@@ -1727,9 +2042,9 @@ extern "C" {
     }
 
     Geometry*
-    GEOSVoronoiDiagram(const Geometry* g, const Geometry* env, double tolerance, int onlyEdges)
+    GEOSVoronoiDiagram(const Geometry* g, const Geometry* env, double tolerance, int flags)
     {
-        return GEOSVoronoiDiagram_r(handle, g, env, tolerance, onlyEdges);
+        return GEOSVoronoiDiagram_r(handle, g, env, tolerance, flags);
     }
 
     int
@@ -1743,4 +2058,20 @@ extern "C" {
                                          cx, cy);
     }
 
+    int
+    GEOSCoverageIsValid(
+        const Geometry* input,
+        double gapWidth,
+        Geometry** invalidEdges)
+    {
+        return GEOSCoverageIsValid_r(handle, input, gapWidth, invalidEdges);
+    }
+
+    Geometry*
+    GEOSCoverageSimplifyVW(const Geometry* input, double tolerance, int preserveBoundary)
+    {
+        return GEOSCoverageSimplifyVW_r(handle, input, tolerance, preserveBoundary);
+    }
+
+
 } /* extern "C" */
