@@ -3,11 +3,11 @@
  * GEOS - Geometry Engine Open Source
  * http://geos.osgeo.org
  *
- * Copyright (C) 2009      Sandro Santilli <strk@keybit.net>
+ * Copyright (C) 2009      Sandro Santilli <strk@kbt.io>
  *
  * This is free software; you can redistribute and/or modify it under
  * the terms of the GNU Lesser General Public Licence as published
- * by the Free Software Foundation. 
+ * by the Free Software Foundation.
  * See the COPYING file for more information.
  *
  **********************************************************************
@@ -25,7 +25,7 @@ namespace algorithm { // geos.algorithm
 // namespace. In GEOS they are hidden and only accessible as singleton
 // from BoundaryNodeRule static const references
 //
-namespace { 
+namespace {
 
 /**
  * A {@link BoundaryNodeRule} specifies that points are in the
@@ -41,14 +41,18 @@ namespace {
  * @author Martin Davis
  * @version 1.7
  */
-class Mod2BoundaryNodeRule : public BoundaryNodeRule
-{
+class Mod2BoundaryNodeRule : public BoundaryNodeRule {
 public:
-	bool isInBoundary(int boundaryCount) const
-	{
-		// the "Mod-2 Rule"
-		return boundaryCount % 2 == 1;
-	}
+    bool
+    isInBoundary(int boundaryCount) const override
+    {
+        // the "Mod-2 Rule"
+        return boundaryCount % 2 == 1;
+    };
+    std::string
+    toString() const override {
+        return "Mod2BoundaryNodeRule";
+    };
 };
 
 
@@ -79,12 +83,16 @@ public:
  * @author Martin Davis
  * @version 1.7
  */
-class EndPointBoundaryNodeRule : public BoundaryNodeRule
-{
-	bool isInBoundary(int boundaryCount) const
-	{
-		return boundaryCount > 0;
-	}
+class EndPointBoundaryNodeRule : public BoundaryNodeRule {
+    bool
+    isInBoundary(int boundaryCount) const override
+    {
+        return boundaryCount > 0;
+    };
+    std::string
+    toString() const override {
+        return "EndPointBoundaryNodeRule";
+    };
 };
 
 /**
@@ -97,12 +105,16 @@ class EndPointBoundaryNodeRule : public BoundaryNodeRule
  * @author Martin Davis
  * @version 1.7
  */
-class MultiValentEndPointBoundaryNodeRule : public BoundaryNodeRule
-{
-	bool isInBoundary(int boundaryCount) const
-	{
-		return boundaryCount > 1;
-	}
+class MultiValentEndPointBoundaryNodeRule : public BoundaryNodeRule {
+    bool
+    isInBoundary(int boundaryCount) const override
+    {
+        return boundaryCount > 1;
+    }
+    std::string
+    toString() const override {
+        return "MultiValentEndPointBoundaryNodeRule";
+    };
 };
 
 /**
@@ -114,12 +126,16 @@ class MultiValentEndPointBoundaryNodeRule : public BoundaryNodeRule
  * @author Martin Davis
  * @version 1.7
  */
-class MonoValentEndPointBoundaryNodeRule : public BoundaryNodeRule
-{
-	bool isInBoundary(int boundaryCount) const
-	{
-		return boundaryCount == 1;
-	}
+class MonoValentEndPointBoundaryNodeRule : public BoundaryNodeRule {
+    bool
+    isInBoundary(int boundaryCount) const override
+    {
+        return boundaryCount == 1;
+    }
+    std::string
+    toString() const override {
+        return "MonoValentEndPointBoundaryNodeRule";
+    };
 };
 
 Mod2BoundaryNodeRule mod2Rule;
@@ -130,31 +146,36 @@ MonoValentEndPointBoundaryNodeRule monoValentRule;
 } // anonymous namespace
 
 //const BoundaryNodeRule& BoundaryNodeRule::MOD2_BOUNDARY_RULE = mod2Rule;
-const BoundaryNodeRule& BoundaryNodeRule::getBoundaryRuleMod2()
+const BoundaryNodeRule&
+BoundaryNodeRule::getBoundaryRuleMod2()
 {
     return mod2Rule;
 }
 
 //const BoundaryNodeRule& BoundaryNodeRule::ENDPOINT_BOUNDARY_RULE = endPointRule;
-const BoundaryNodeRule& BoundaryNodeRule::getBoundaryEndPoint()
+const BoundaryNodeRule&
+BoundaryNodeRule::getBoundaryEndPoint()
 {
     return endPointRule;
 }
 
 //const BoundaryNodeRule& BoundaryNodeRule::MULTIVALENT_ENDPOINT_BOUNDARY_RULE = multiValentRule;
-const BoundaryNodeRule& BoundaryNodeRule::getBoundaryMultivalentEndPoint()
+const BoundaryNodeRule&
+BoundaryNodeRule::getBoundaryMultivalentEndPoint()
 {
     return multiValentRule;
 }
 
 //const BoundaryNodeRule& BoundaryNodeRule::MONOVALENT_ENDPOINT_BOUNDARY_RULE = monoValentRule;
-const BoundaryNodeRule& BoundaryNodeRule::getBoundaryMonovalentEndPoint()
+const BoundaryNodeRule&
+BoundaryNodeRule::getBoundaryMonovalentEndPoint()
 {
     return monoValentRule;
 }
 
 //const BoundaryNodeRule& BoundaryNodeRule::OGC_SFS_BOUNDARY_RULE = BoundaryNodeRule::MOD2_BOUNDARY_RULE;
-const BoundaryNodeRule& BoundaryNodeRule::getBoundaryOGCSFS()
+const BoundaryNodeRule&
+BoundaryNodeRule::getBoundaryOGCSFS()
 {
     return getBoundaryRuleMod2();
 }
