@@ -39,19 +39,22 @@
 #include <geos/algorithm/InteriorPointLine.h>
 #include <geos/algorithm/InteriorPointArea.h>
 #include <geos/algorithm/ConvexHull.h>
+#include <geos/algorithm/locate/SimplePointInAreaLocator.h>
+#include <geos/geom/prep/PreparedGeometryFactory.h>
 #include <geos/operation/intersection/Rectangle.h>
 #include <geos/operation/intersection/RectangleIntersection.h>
 #include <geos/operation/predicate/RectangleContains.h>
 #include <geos/operation/predicate/RectangleIntersects.h>
 #include <geos/operation/relate/RelateOp.h>
+#include <geos/operation/relateng/RelateNG.h>
+#include <geos/operation/relateng/RelatePredicate.h>
 #include <geos/operation/valid/IsValidOp.h>
-#include <geos/operation/overlay/OverlayOp.h>
 #include <geos/operation/union/UnaryUnionOp.h>
-#include <geos/operation/overlay/snap/SnapIfNeededOverlayOp.h>
 #include <geos/operation/buffer/BufferOp.h>
 #include <geos/operation/distance/DistanceOp.h>
-#include <geos/operation/IsSimpleOp.h>
+#include <geos/operation/valid/IsSimpleOp.h>
 #include <geos/operation/overlayng/OverlayNGRobust.h>
+#include <geos/operation/overlayng/OverlayUtil.h>
 #include <geos/io/WKBWriter.h>
 #include <geos/io/WKTWriter.h>
 #include <geos/version.h>
@@ -71,16 +74,16 @@
 
 #define SHORTCIRCUIT_PREDICATES 1
 //#define USE_RECTANGLE_INTERSECTION 1
+#define USE_RELATENG 1
 
-using namespace std;
 using namespace geos::algorithm;
 using namespace geos::operation::valid;
 using namespace geos::operation::relate;
 using namespace geos::operation::buffer;
-using namespace geos::operation::overlay;
-using namespace geos::operation::overlay::snap;
 using namespace geos::operation::distance;
 using namespace geos::operation;
+using geos::operation::overlayng::OverlayNG;
+
 
 namespace geos {
 namespace geom { // geos::geom
@@ -89,7 +92,7 @@ namespace geom { // geos::geom
 /*
  * Return current GEOS version
  */
-string
+std::string
 geosversion()
 {
     return GEOS_VERSION;
@@ -99,7 +102,7 @@ geosversion()
  * Return the version of JTS this GEOS
  * release has been ported from.
  */
-string
+std::string
 jtsport()
 {
     return GEOS_JTS_PORT;
@@ -109,7 +112,6 @@ Geometry::GeometryChangedFilter Geometry::geometryChangedFilter;
 
 Geometry::Geometry(const GeometryFactory* newFactory)
     :
-    envelope(nullptr),
     _factory(newFactory),
     _userData(nullptr)
 {
@@ -126,21 +128,14 @@ Geometry::Geometry(const Geometry& geom)
     _factory(geom._factory),
     _userData(nullptr)
 {
-    if(geom.envelope.get()) {
-        envelope.reset(new Envelope(*(geom.envelope)));
-    }
-    //factory=geom.factory;
-    //envelope(new Envelope(*(geom.envelope.get())));
-    //SRID=geom.getSRID();
-    //_userData=NULL;
     _factory->addRef();
 }
 
 bool
 Geometry::hasNullElements(const CoordinateSequence* list)
 {
-    size_t npts = list->getSize();
-    for(size_t i = 0; i < npts; ++i) {
+    std::size_t npts = list->getSize();
+    for(std::size_t i = 0; i < npts; ++i) {
         if(list->getAt(i).isNull()) {
             return true;
         }
@@ -152,19 +147,7 @@ Geometry::hasNullElements(const CoordinateSequence* list)
 bool
 Geometry::isWithinDistance(const Geometry* geom, double cDistance) const
 {
-    const Envelope* env0 = getEnvelopeInternal();
-    const Envelope* env1 = geom->getEnvelopeInternal();
-    double envDist = env0->distance(*env1);
-
-    if(envDist > cDistance) {
-        return false;
-    }
-    // NOTE: this could be implemented more efficiently
-    double geomDist = distance(geom);
-    if(geomDist > cDistance) {
-        return false;
-    }
-    return true;
+    return DistanceOp::isWithinDistance(*this, *geom, cDistance);
 }
 
 /*public*/
@@ -173,7 +156,7 @@ Geometry::getCentroid() const
 {
     Coordinate centPt;
     if(! getCentroid(centPt)) {
-        return nullptr;
+        return getFactory()->createPoint(getCoordinateDimension());
     }
 
     // We don't use createPointFromInternalCoord here
@@ -181,9 +164,37 @@ Geometry::getCentroid() const
     return std::unique_ptr<Point>(getFactory()->createPoint(centPt));
 }
 
+/* public */
+bool
+Geometry::isMixedDimension() const
+{
+    Dimension::DimensionType baseDim = Dimension::DONTCARE;
+    return isMixedDimension(&baseDim);
+}
+
+/* public */
+bool
+Geometry::isMixedDimension(Dimension::DimensionType* baseDim) const
+{
+    if (isCollection()) {
+        for (std::size_t i = 0; i < getNumGeometries(); i++) {
+            if (getGeometryN(i)->isMixedDimension(baseDim))
+                return true;
+        }
+        return false;
+    }
+    else {
+        if (*baseDim == Dimension::DONTCARE) {
+            *baseDim = getDimension();
+            return false;
+        }
+        return *baseDim != getDimension();
+    }
+}
+
 /*public*/
 bool
-Geometry::getCentroid(Coordinate& ret) const
+Geometry::getCentroid(CoordinateXY& ret) const
 {
     if(isEmpty()) {
         return false;
@@ -198,24 +209,26 @@ Geometry::getCentroid(Coordinate& ret) const
 std::unique_ptr<Point>
 Geometry::getInteriorPoint() const
 {
+    geos::util::ensureNoCurvedComponents(this);
+
     Coordinate interiorPt;
     int dim = getDimension();
     if(dim == 0) {
         InteriorPointPoint intPt(this);
         if(! intPt.getInteriorPoint(interiorPt)) {
-            return nullptr;
+            return getFactory()->createPoint(getCoordinateDimension()); // POINT EMPTY
         }
     }
     else if(dim == 1) {
         InteriorPointLine intPt(this);
         if(! intPt.getInteriorPoint(interiorPt)) {
-            return nullptr;
+            return getFactory()->createPoint(getCoordinateDimension()); // POINT EMPTY
         }
     }
     else {
         InteriorPointArea intPt(this);
         if(! intPt.getInteriorPoint(interiorPt)) {
-            return nullptr;
+            return getFactory()->createPoint(getCoordinateDimension()); // POINT EMPTY
         }
     }
     std::unique_ptr<Point> p(getFactory()->createPointFromInternalCoord(&interiorPt, this));
@@ -233,18 +246,6 @@ Geometry::geometryChanged()
     apply_rw(&geometryChangedFilter);
 }
 
-/**
- * Notifies this Geometry that its Coordinates have been changed by an external
- * party. When geometryChanged is called, this method will be called for
- * this Geometry and its component Geometries.
- * @see apply(GeometryComponentFilter *)
- */
-void
-Geometry::geometryChangedAction()
-{
-    envelope.reset(nullptr);
-}
-
 bool
 Geometry::isValid() const
 {
@@ -257,27 +258,14 @@ Geometry::getEnvelope() const
     return std::unique_ptr<Geometry>(getFactory()->toGeometry(getEnvelopeInternal()));
 }
 
-const Envelope*
-Geometry::getEnvelopeInternal() const
-{
-    if(!envelope.get()) {
-        envelope = computeEnvelopeInternal();
-    }
-    return envelope.get();
-}
-
 bool
 Geometry::disjoint(const Geometry* g) const
 {
-#ifdef SHORTCIRCUIT_PREDICATES
-    // short-circuit test
-    if(! getEnvelopeInternal()->intersects(g->getEnvelopeInternal())) {
-        return true;
-    }
+#if USE_RELATENG
+    return operation::relateng::RelateNG::disjoint(this, g);
+#else
+    return !intersects(g);
 #endif
-    unique_ptr<IntersectionMatrix> im(relate(g));
-    bool res = im->isDisjoint();
-    return res;
 }
 
 bool
@@ -289,9 +277,14 @@ Geometry::touches(const Geometry* g) const
         return false;
     }
 #endif
-    unique_ptr<IntersectionMatrix> im(relate(g));
+
+#if USE_RELATENG
+    return operation::relateng::RelateNG::touches(this, g);
+#else
+    std::unique_ptr<IntersectionMatrix> im(relate(g));
     bool res = im->isTouches(getDimension(), g->getDimension());
     return res;
+#endif
 }
 
 bool
@@ -322,23 +315,43 @@ Geometry::intersects(const Geometry* g) const
 
     // optimization for rectangle arguments
     if(isRectangle()) {
-        const Polygon* p = dynamic_cast<const Polygon*>(this);
+        const Polygon* p = detail::down_cast<const Polygon*>(this);
         return predicate::RectangleIntersects::intersects(*p, *g);
     }
     if(g->isRectangle()) {
-        const Polygon* p = dynamic_cast<const Polygon*>(g);
+        const Polygon* p = detail::down_cast<const Polygon*>(g);
         return predicate::RectangleIntersects::intersects(*p, *this);
     }
 
-    unique_ptr<IntersectionMatrix> im(relate(g));
-    bool res = im->isIntersects();
-    return res;
+    auto typ = getGeometryTypeId();
+    if (typ == GEOS_CURVEPOLYGON && g->getGeometryTypeId() == GEOS_POINT) {
+        auto loc = locate::SimplePointInAreaLocator::locatePointInSurface(*g->getCoordinate(), *detail::down_cast<const Surface*>(this));
+        return loc != Location::EXTERIOR;
+    } else if (typ == GEOS_POINT && g->getGeometryTypeId() == GEOS_CURVEPOLYGON) {
+        auto loc = locate::SimplePointInAreaLocator::locatePointInSurface(*getCoordinate(), *detail::down_cast<const Surface*>(g));
+        return loc != Location::EXTERIOR;
+    }
+
+#if USE_RELATENG
+    return operation::relateng::RelateNG::intersects(this, g);
+#else
+    if (typ == GEOS_GEOMETRYCOLLECTION) {
+        auto im = relate(g);
+        bool res = im->isIntersects();
+        return res;
+    } else {
+        return prep::PreparedGeometryFactory::prepare(this)->intersects(g);
+    }
+#endif
 }
 
 /*public*/
 bool
 Geometry::covers(const Geometry* g) const
 {
+#if USE_RELATENG
+    return operation::relateng::RelateNG::covers(this, g);
+#else
     // optimization - lower dimension cannot cover areas
     if(g->getDimension() == 2 && getDimension() < 2) {
         return false;
@@ -364,34 +377,59 @@ Geometry::covers(const Geometry* g) const
         return true;
     }
 
-    unique_ptr<IntersectionMatrix> im(relate(g));
+    std::unique_ptr<IntersectionMatrix> im(relate(g));
     return im->isCovers();
+#endif
 }
 
+/*public*/
+bool
+Geometry::coveredBy(const Geometry* g) const
+{
+#if USE_RELATENG
+    return operation::relateng::RelateNG::coveredBy(this, g);
+#else
+    return covers(g, this);
+#endif
+}
 
 bool
 Geometry::crosses(const Geometry* g) const
 {
+#if USE_RELATENG
+    return operation::relateng::RelateNG::crosses(this, g);
+#else
+
 #ifdef SHORTCIRCUIT_PREDICATES
     // short-circuit test
     if(! getEnvelopeInternal()->intersects(g->getEnvelopeInternal())) {
         return false;
     }
 #endif
-    unique_ptr<IntersectionMatrix> im(relate(g));
+    std::unique_ptr<IntersectionMatrix> im(relate(g));
     bool res = im->isCrosses(getDimension(), g->getDimension());
     return res;
+
+#endif
 }
 
 bool
 Geometry::within(const Geometry* g) const
 {
+#if USE_RELATENG
+    return operation::relateng::RelateNG::within(this, g);
+#else
     return g->contains(this);
+#endif
 }
 
 bool
 Geometry::contains(const Geometry* g) const
 {
+#if USE_RELATENG
+    return operation::relateng::RelateNG::contains(this, g);
+#else
+
     // optimization - lower dimension cannot contain areas
     if(g->getDimension() == 2 && getDimension() < 2) {
         return false;
@@ -413,7 +451,7 @@ Geometry::contains(const Geometry* g) const
 
     // optimization for rectangle arguments
     if(isRectangle()) {
-        const Polygon* p = dynamic_cast<const Polygon*>(this);
+        const Polygon* p = detail::down_cast<const Polygon*>(this);
         return predicate::RectangleContains::contains(*p, *g);
     }
     // Incorrect: contains is not commutative
@@ -421,36 +459,51 @@ Geometry::contains(const Geometry* g) const
     //	return predicate::RectangleContains::contains((const Polygon&)*g, *this);
     //}
 
-    unique_ptr<IntersectionMatrix> im(relate(g));
+    std::unique_ptr<IntersectionMatrix> im(relate(g));
     bool res = im->isContains();
     return res;
+#endif
 }
 
 bool
 Geometry::overlaps(const Geometry* g) const
 {
+#if USE_RELATENG
+    return operation::relateng::RelateNG::overlaps(this, g);
+#else
+
 #ifdef SHORTCIRCUIT_PREDICATES
     // short-circuit test
     if(! getEnvelopeInternal()->intersects(g->getEnvelopeInternal())) {
         return false;
     }
 #endif
-    unique_ptr<IntersectionMatrix> im(relate(g));
+    std::unique_ptr<IntersectionMatrix> im(relate(g));
     bool res = im->isOverlaps(getDimension(), g->getDimension());
     return res;
+
+#endif
 }
 
 bool
-Geometry::relate(const Geometry* g, const string& intersectionPattern) const
+Geometry::relate(const Geometry* g, const std::string& intersectionPattern) const
 {
-    unique_ptr<IntersectionMatrix> im(relate(g));
+#if USE_RELATENG
+    return operation::relateng::RelateNG::relate(this, g, intersectionPattern);
+#else
+    std::unique_ptr<IntersectionMatrix> im(relate(g));
     bool res = im->matches(intersectionPattern);
     return res;
+#endif
 }
 
 bool
 Geometry::equals(const Geometry* g) const
 {
+#if USE_RELATENG
+    return operation::relateng::RelateNG::equalsTopo(this, g);
+#else
+
 #ifdef SHORTCIRCUIT_PREDICATES
     // short-circuit test
     if(! getEnvelopeInternal()->equals(g->getEnvelopeInternal())) {
@@ -465,18 +518,29 @@ Geometry::equals(const Geometry* g) const
         return isEmpty();
     }
 
-    unique_ptr<IntersectionMatrix> im(relate(g));
+    std::unique_ptr<IntersectionMatrix> im(relate(g));
     bool res = im->isEquals(getDimension(), g->getDimension());
     return res;
+#endif
 }
 
 std::unique_ptr<IntersectionMatrix>
 Geometry::relate(const Geometry* other) const
 {
+#if USE_RELATENG
+    return operation::relateng::RelateNG::relate(this, other);
+#else
     return RelateOp::relate(this, other);
+#endif
 }
 
-string
+std::unique_ptr<IntersectionMatrix>
+Geometry::relate(const Geometry& other) const
+{
+    return relate(&other);
+}
+
+std::string
 Geometry::toString() const
 {
     return toText();
@@ -490,10 +554,13 @@ operator<< (std::ostream& os, const Geometry& geom)
     return os;
 }
 
-string
+std::string
 Geometry::toText() const
 {
     io::WKTWriter writer;
+    // To enable "GEOS<3.12" outputs, uncomment the following:
+    // writer.setTrim(false);
+    // writer.setOutputDimension(2);
     return writer.write(this);
 }
 
@@ -528,11 +595,6 @@ Geometry::intersection(const Geometry* other) const
      * TODO: MD - add optimization for P-A case using Point-In-Polygon
      */
 
-    // special case: if one input is empty ==> empty
-    if(isEmpty() || other->isEmpty()) {
-        return OverlayOp::createEmptyResult(OverlayOp::opINTERSECTION, this, other, getFactory());
-    }
-
 #ifdef USE_RECTANGLE_INTERSECTION
     // optimization for rectangle arguments
     using operation::intersection::Rectangle;
@@ -551,61 +613,13 @@ Geometry::intersection(const Geometry* other) const
     }
 #endif
 
-    return HeuristicOverlay(this, other, OverlayOp::opINTERSECTION);
+    return HeuristicOverlay(this, other, OverlayNG::INTERSECTION);
 }
 
 std::unique_ptr<Geometry>
 Geometry::Union(const Geometry* other) const
 {
-    // handle empty geometry cases
-    if(isEmpty() || other->isEmpty() ) {
-      if(isEmpty() && other->isEmpty() ) {
-        return OverlayOp::createEmptyResult(OverlayOp::opUNION, this, other, getFactory());
-      }
-      // special case: if one input is empty ==> other input
-      if(isEmpty()) return other->clone();
-      if(other->isEmpty()) return clone();
-    }
-
-#ifdef SHORTCIRCUIT_PREDICATES
-    // if envelopes are disjoint return a MULTI geom or
-    // a geometrycollection
-    if(! getEnvelopeInternal()->intersects(other->getEnvelopeInternal())) {
-//cerr<<"SHORTCIRCUITED-UNION engaged"<<endl;
-        const GeometryCollection* coll;
-
-        size_t ngeomsThis = getNumGeometries();
-        size_t ngeomsOther = other->getNumGeometries();
-
-        // Allocated for ownership transfer
-        vector<Geometry*>* v = new vector<Geometry*>();
-        v->reserve(ngeomsThis + ngeomsOther);
-
-
-        if(nullptr != (coll = dynamic_cast<const GeometryCollection*>(this))) {
-            for(size_t i = 0; i < ngeomsThis; ++i) {
-                v->push_back(coll->getGeometryN(i)->clone().release());
-            }
-        }
-        else {
-            v->push_back(this->clone().release());
-        }
-
-        if(nullptr != (coll = dynamic_cast<const GeometryCollection*>(other))) {
-            for(size_t i = 0; i < ngeomsOther; ++i) {
-                v->push_back(coll->getGeometryN(i)->clone().release());
-            }
-        }
-        else {
-            v->push_back(other->clone().release());
-        }
-
-        std::unique_ptr<Geometry>out(_factory->buildGeometry(v));
-        return out;
-    }
-#endif
-
-    return HeuristicOverlay(this, other, OverlayOp::opUNION);
+    return HeuristicOverlay(this, other, OverlayNG::UNION);
 }
 
 /* public */
@@ -613,77 +627,19 @@ Geometry::Ptr
 Geometry::Union() const
 {
     using geos::operation::geounion::UnaryUnionOp;
-#ifdef DISABLE_OVERLAYNG
     return UnaryUnionOp::Union(*this);
-#else
-    return operation::overlayng::OverlayNGRobust::Union(this);
-#endif
 }
 
 std::unique_ptr<Geometry>
 Geometry::difference(const Geometry* other) const
-//throw(IllegalArgumentException *)
 {
-    // special case: if A.isEmpty ==> empty; if B.isEmpty ==> A
-    if(isEmpty()) {
-        return OverlayOp::createEmptyResult(OverlayOp::opDIFFERENCE, this, other, getFactory());
-    }
-    if(other->isEmpty()) {
-        return clone();
-    }
-
-    return HeuristicOverlay(this, other, OverlayOp::opDIFFERENCE);
+    return HeuristicOverlay(this, other, OverlayNG::DIFFERENCE);
 }
 
 std::unique_ptr<Geometry>
 Geometry::symDifference(const Geometry* other) const
 {
-    // handle empty geometry cases
-    if(isEmpty() || other->isEmpty() ) {
-      if(isEmpty() && other->isEmpty() ) {
-        return OverlayOp::createEmptyResult(OverlayOp::opSYMDIFFERENCE, this, other, getFactory());
-      }
-      // special case: if either input is empty ==> other input
-      if(isEmpty()) return other->clone();
-      if(other->isEmpty()) return clone();
-    }
-
-    // if envelopes are disjoint return a MULTI geom or
-    // a geometrycollection
-    if(! getEnvelopeInternal()->intersects(other->getEnvelopeInternal())) {
-        const GeometryCollection* coll;
-
-        size_t ngeomsThis = getNumGeometries();
-        size_t ngeomsOther = other->getNumGeometries();
-
-        // Allocated for ownership transfer
-        vector<Geometry*>* v = new vector<Geometry*>();
-        v->reserve(ngeomsThis + ngeomsOther);
-
-
-        if(nullptr != (coll = dynamic_cast<const GeometryCollection*>(this))) {
-            for(size_t i = 0; i < ngeomsThis; ++i) {
-                v->push_back(coll->getGeometryN(i)->clone().release());
-            }
-        }
-        else {
-            v->push_back(this->clone().release());
-        }
-
-        if(nullptr != (coll = dynamic_cast<const GeometryCollection*>(other))) {
-            for(size_t i = 0; i < ngeomsOther; ++i) {
-                v->push_back(coll->getGeometryN(i)->clone().release());
-            }
-        }
-        else {
-            v->push_back(other->clone().release());
-        }
-
-        return std::unique_ptr<Geometry>(_factory->buildGeometry(v));
-    }
-
-    return HeuristicOverlay(this, other, OverlayOp::opSYMDIFFERENCE);
-
+    return HeuristicOverlay(this, other, OverlayNG::SYMDIFFERENCE);
 }
 
 int
@@ -738,78 +694,6 @@ Geometry::GeometryChangedFilter::filter_rw(Geometry* geom)
     geom->geometryChangedAction();
 }
 
-int
-Geometry::compare(vector<Coordinate> a, vector<Coordinate> b) const
-{
-    size_t i = 0;
-    size_t j = 0;
-    while(i < a.size() && j < b.size()) {
-        Coordinate& aCoord = a[i];
-        Coordinate& bCoord = b[j];
-        int comparison = aCoord.compareTo(bCoord);
-        if(comparison != 0) {
-            return comparison;
-        }
-        i++;
-        j++;
-    }
-    if(i < a.size()) {
-        return 1;
-    }
-    if(j < b.size()) {
-        return -1;
-    }
-    return 0;
-}
-
-int
-Geometry::compare(vector<Geometry*> a, vector<Geometry*> b) const
-{
-    size_t i = 0;
-    size_t j = 0;
-    while(i < a.size() && j < b.size()) {
-        Geometry* aGeom = a[i];
-        Geometry* bGeom = b[j];
-        int comparison = aGeom->compareTo(bGeom);
-        if(comparison != 0) {
-            return comparison;
-        }
-        i++;
-        j++;
-    }
-    if(i < a.size()) {
-        return 1;
-    }
-    if(j < b.size()) {
-        return -1;
-    }
-    return 0;
-}
-
-int
-Geometry::compare(const std::vector<std::unique_ptr<Geometry>> & a,
-        const std::vector<std::unique_ptr<Geometry>> & b) const
-{
-    size_t i = 0;
-    size_t j = 0;
-    while(i < a.size() && j < b.size()) {
-        Geometry* aGeom = a[i].get();
-        Geometry* bGeom = b[j].get();
-        int comparison = aGeom->compareTo(bGeom);
-        if(comparison != 0) {
-            return comparison;
-        }
-        i++;
-        j++;
-    }
-    if(i < a.size()) {
-        return 1;
-    }
-    if(j < b.size()) {
-        return -1;
-    }
-    return 0;
-}
 
 /**
  *  Returns the minimum distance between this Geometry
@@ -869,7 +753,7 @@ GeometryGreaterThen::operator()(const Geometry* first, const Geometry* second)
 }
 
 bool
-Geometry::equal(const Coordinate& a, const Coordinate& b,
+Geometry::equal(const CoordinateXY& a, const CoordinateXY& b,
                 double tolerance) const
 {
     if(tolerance == 0) {
@@ -906,7 +790,7 @@ Geometry::apply_rw(GeometryComponentFilter* filter)
 bool
 Geometry::isSimple() const
 {
-    operation::IsSimpleOp op(*this);
+    operation::valid::IsSimpleOp op(*this);
     return op.isSimple();
 }
 
@@ -917,6 +801,11 @@ Geometry::getPrecisionModel() const
     return _factory->getPrecisionModel();
 }
 
+bool
+Geometry::hasCurvedComponents() const {
+    return false;
+}
+
 } // namespace geos::geom
 } // namespace geos
 
