@@ -18,12 +18,11 @@
  **********************************************************************/
 
 #include <geos/index/quadtree/IntervalSize.h>
-#include <geos/index/quadtree/DoubleBits.h>
 
 #include <algorithm>
 #include <cmath>
 
-using namespace std;
+
 
 namespace geos {
 namespace index { // geos.index
@@ -38,9 +37,11 @@ IntervalSize::isZeroWidth(double mn, double mx)
         return true;
     }
 
-    double maxAbs = max(fabs(mn), fabs(mx));
+    double maxAbs = std::max(std::fabs(mn), std::fabs(mx));
     double scaledInterval = width / maxAbs;
-    int level = DoubleBits::exponent(scaledInterval);
+    int level;
+    frexp(scaledInterval, &level);
+    level -= 1;
     return level <= MIN_BINARY_EXPONENT;
 }
 
