@@ -35,5 +35,19 @@ void object::test<1>()
     ensure_equals(0, GEOSOverlaps(geom3_, geom2_));
 }
 
+template<>
+template<>
+void object::test<2>()
+{
+    geom1_ = fromWKT("CIRCULARSTRING (0 0, 1 1, 2 0)");
+    geom2_ = fromWKT("LINESTRING (1 0, 2 1)");
+
+    ensure(geom1_);
+    ensure(geom2_);
+
+    ensure_equals("curved geometry not supported", GEOSOverlaps(geom1_, geom2_), 2);
+    ensure_equals("curved geometry not supported", GEOSOverlaps(geom2_, geom1_), 2);
+}
+
 } // namespace tut
 
