@@ -5,9 +5,6 @@
 // geos
 #include <geos_c.h>
 // std
-#include <cstdarg>
-#include <cstdio>
-#include <cstdlib>
 #include <cstring>
 
 #include "capi_test_utils.h"
