@@ -6,10 +6,6 @@
 #include <geos_c.h>
 // std
 #include <cctype>
-#include <cstdarg>
-#include <cstdio>
-#include <cstdlib>
-#include <cstring>
 
 #include "capi_test_utils.h"
 
