@@ -8,9 +8,7 @@
 #include <geos/geom/GeometryFactory.h>
 #include <geos/geom/GeometryCollection.h>
 #include <geos/geom/Coordinate.h>
-#include <geos/geom/CoordinateArraySequence.h>
-#include <geos/geom/CoordinateArraySequenceFactory.h>
-#include <geos/geom/CoordinateSequenceFactory.h>
+#include <geos/geom/CoordinateSequence.h>
 #include <geos/geom/Dimension.h>
 #include <geos/geom/LinearRing.h>
 #include <geos/geom/LineString.h>
@@ -22,10 +20,18 @@
 #include <geos/geom/PrecisionModel.h>
 #include <geos/io/WKTReader.h>
 #include <geos/util/IllegalArgumentException.h>
+#include <geos/util.h>
 // std
 #include <vector>
 #include <cstring> // std::size_t
 
+using geos::geom::Coordinate;
+using geos::geom::CoordinateXY;
+using geos::geom::CoordinateXYM;
+using geos::geom::CoordinateXYZM;
+using geos::geom::GeometryFactory;
+using geos::geom::PrecisionModel;
+
 /*!
  * \brief
  * Write brief comment for tut here.
@@ -81,16 +87,15 @@ template<>
 void object::test<1>
 ()
 {
-    using geos::geom::GeometryFactory;
     GeometryFactory::Ptr gf = GeometryFactory::create();
 
     ensure_equals(gf->getSRID(), 0);
-    ensure_equals(gf->getPrecisionModel()->getType(), geos::geom::PrecisionModel::FLOATING);
+    ensure_equals(gf->getPrecisionModel()->getType(), PrecisionModel::FLOATING);
 
     auto geo = gf->createEmptyGeometry();
     ensure("createEmptyGeometry() returned null pointer.", geo != nullptr);
     ensure_equals(geo->getSRID(), gf->getSRID());
-    ensure_equals(geo->getPrecisionModel()->getType(), geos::geom::PrecisionModel::FLOATING);
+    ensure_equals(geo->getPrecisionModel()->getType(), PrecisionModel::FLOATING);
 }
 
 // Test of user's constructor
@@ -99,28 +104,17 @@ template<>
 void object::test<2>
 ()
 {
-    using geos::geom::GeometryFactory;
-    using geos::geom::PrecisionModel;
-    using geos::geom::CoordinateArraySequenceFactory;
+    PrecisionModel pm(1.0);
+    GeometryFactory::Ptr gf = GeometryFactory::create(&pm, srid_);
 
-    CoordinateArraySequenceFactory csf;
-
-    {
-        PrecisionModel pm(1.0);
-        GeometryFactory::Ptr gf = GeometryFactory::create(&pm, srid_, &csf);
-
-        ensure_equals(gf->getSRID(), srid_);
-        ensure_equals(gf->getPrecisionModel()->getType(), geos::geom::PrecisionModel::FIXED);
-
-        ensure_equals(&csf, gf->getCoordinateSequenceFactory());
-
-        auto geo = gf->createEmptyGeometry();
-        ensure("createEmptyGeometry() returned null pointer.", geo != nullptr);
-        ensure_equals(geo->getSRID(), gf->getSRID());
-        ensure_equals(geo->getPrecisionModel()->getType(), geos::geom::PrecisionModel::FIXED);
-    }
-    // csf lifetime must exceed lifetime of the GeometryFactory instance
+    ensure_equals(gf->getSRID(), srid_);
+    ensure_equals(gf->getPrecisionModel()->getType(), PrecisionModel::FIXED);
 
+    auto geo = gf->createEmptyGeometry();
+    ensure("createEmptyGeometry() returned null pointer.", geo != nullptr);
+    ensure_equals(geo->getSRID(), gf->getSRID());
+    ensure_equals(geo->getSRID(), srid_);
+    ensure_equals(geo->getPrecisionModel()->getType(), PrecisionModel::FIXED);
 } // test<2>
 
 // Test of user's constructor
@@ -129,23 +123,15 @@ template<>
 void object::test<3>
 ()
 {
-    using geos::geom::GeometryFactory;
-    using geos::geom::CoordinateArraySequenceFactory;
+    GeometryFactory::Ptr gf = GeometryFactory::create();
 
-    CoordinateArraySequenceFactory csf;
+    ensure_equals(gf->getSRID(), 0);
+    ensure_equals(gf->getPrecisionModel()->getType(), PrecisionModel::FLOATING);
 
-    {
-        GeometryFactory::Ptr gf = GeometryFactory::create(&csf);
-
-        ensure_equals(gf->getSRID(), 0);
-        ensure_equals(gf->getPrecisionModel()->getType(), geos::geom::PrecisionModel::FLOATING);
-
-        auto geo = gf->createEmptyGeometry();
-        ensure("createEmptyGeometry() returned null pointer.", geo != nullptr);
-        ensure_equals(geo->getSRID(), gf->getSRID());
-        ensure_equals(geo->getPrecisionModel()->getType(), geos::geom::PrecisionModel::FLOATING);
-    }
-    // csf lifetime must exceed lifetime of the GeometryFactory instance
+    auto geo = gf->createEmptyGeometry();
+    ensure("createEmptyGeometry() returned null pointer.", geo != nullptr);
+    ensure_equals(geo->getSRID(), gf->getSRID());
+    ensure_equals(geo->getPrecisionModel()->getType(), PrecisionModel::FLOATING);
 }
 
 // Test of user's constructor
@@ -154,9 +140,6 @@ template<>
 void object::test<4>
 ()
 {
-    using geos::geom::PrecisionModel;
-    using geos::geom::GeometryFactory;
-
     PrecisionModel pm(PrecisionModel::FIXED);
     GeometryFactory::Ptr gf(GeometryFactory::create(&pm));
 
@@ -175,9 +158,6 @@ template<>
 void object::test<5>
 ()
 {
-    using geos::geom::PrecisionModel;
-    using geos::geom::GeometryFactory;
-
     PrecisionModel pm(PrecisionModel::FIXED);
     GeometryFactory::Ptr gf(GeometryFactory::create(&pm, srid_));
 
@@ -196,7 +176,6 @@ template<>
 void object::test<6>
 ()
 {
-    using geos::geom::GeometryFactory;
     GeometryFactory::Ptr gf(GeometryFactory::create(*factory_));
 
     ensure_equals(factory_->getSRID(), gf->getSRID());
@@ -254,11 +233,13 @@ void object::test<8>
     ensure(geo->isEmpty());
 
     ensure_equals(pt->getGeometryTypeId(), geos::geom::GEOS_POINT);
+    ensure_equals(pt->getCoordinateDimension(), 2u);
     ensure_equals(pt->getDimension(), geos::geom::Dimension::P);
     ensure_equals(pt->getBoundaryDimension(), geos::geom::Dimension::False);
     ensure_equals(pt->getNumPoints(), 0u);
     ensure_equals(pt->getLength(), 0.0);
     ensure_equals(pt->getArea(), 0.0);
+    ensure_equals(pt->getSRID(), srid_);
 }
 
 // Test of createPoint(const Coordinate &coordinate) const
@@ -269,7 +250,7 @@ void object::test<9>
 {
     geos::geom::Coordinate coord(x_, y_, z_);
 
-    PointPtr pt = factory_->createPoint(coord);
+    auto pt = factory_->createPoint(coord);
 
     ensure("createPoint() returned null pointer.", pt != nullptr);
     ensure("createPoint() returned empty point.", !pt->isEmpty());
@@ -277,11 +258,11 @@ void object::test<9>
     ensure(pt->isValid());
     ensure(pt->getCoordinate() != nullptr);
 
-    CoordinateCPtr pcoord = pt->getCoordinate();
-    ensure(pcoord != nullptr);
-    ensure_equals(pcoord->x, x_);
-    ensure_equals(pcoord->y, y_);
-    ensure_equals(pcoord->z, z_);
+    Coordinate pcoord;
+    pt->getCoordinatesRO()->getAt(0, pcoord);
+    ensure_equals(pcoord.x, x_);
+    ensure_equals(pcoord.y, y_);
+    ensure_equals(pcoord.z, z_);
 
     std::unique_ptr<geos::geom::Geometry> geo;
     geo = pt->getEnvelope();
@@ -300,18 +281,16 @@ void object::test<9>
     ensure(geo != nullptr);
     ensure(!geo->isEmpty());
 
+    ensure_equals(pt->getCoordinateDimension(), 3u);
     ensure_equals(pt->getGeometryTypeId(), geos::geom::GEOS_POINT);
     ensure_equals(pt->getDimension(), geos::geom::Dimension::P);
     ensure_equals(pt->getBoundaryDimension(), geos::geom::Dimension::False);
     ensure_equals(pt->getNumPoints(), 1u);
     ensure_equals(pt->getLength(), 0.0);
     ensure_equals(pt->getArea(), 0.0);
-
-    // FREE MEMORY
-    factory_->destroyGeometry(pt);
 }
 
-// Test of createPoint(CoordinateSequence *coordinates) const
+// Test of createPoint(std::unique_ptr<CoordinateSequence>&& coordinates) const
 template<>
 template<>
 void object::test<10>
@@ -319,12 +298,12 @@ void object::test<10>
 {
     geos::geom::Coordinate coord(x_, y_, z_);
 
-    CoordArrayPtr sequence = new geos::geom::CoordinateArraySequence();
+    auto sequence = geos::detail::make_unique<CoordinateSequence>();
 
     ensure("sequence is null pointer.", sequence != nullptr);
     sequence->add(coord);
 
-    PointPtr pt = factory_->createPoint(sequence);
+    auto pt = factory_->createPoint(std::move(sequence));
 
     ensure("createPoint() returned null pointer.", pt != nullptr);
     ensure("createPoint() returned empty point.", !pt->isEmpty());
@@ -332,11 +311,11 @@ void object::test<10>
     ensure(pt->isValid());
     ensure(pt->getCoordinate() != nullptr);
 
-    CoordinateCPtr pcoord = pt->getCoordinate();
-    ensure(pcoord != nullptr);
-    ensure_equals(pcoord->x, x_);
-    ensure_equals(pcoord->y, y_);
-    ensure_equals(pcoord->z, z_);
+    Coordinate pcoord;
+    pt->getCoordinatesRO()->getAt(0, pcoord);
+    ensure_equals(pcoord.x, x_);
+    ensure_equals(pcoord.y, y_);
+    ensure_equals(pcoord.z, z_);
 
     std::unique_ptr<geos::geom::Geometry> geo;
     geo = pt->getEnvelope();
@@ -361,9 +340,6 @@ void object::test<10>
     ensure_equals(pt->getNumPoints(), 1u);
     ensure_equals(pt->getLength(), 0.0);
     ensure_equals(pt->getArea(), 0.0);
-
-    // FREE MEMORY
-    factory_->destroyGeometry(pt);
 }
 
 // Test of createPoint(const CoordinateSequence &coordinates) const
@@ -374,10 +350,10 @@ void object::test<11>
 {
     geos::geom::Coordinate coord(x_, y_, z_);
 
-    geos::geom::CoordinateArraySequence sequence;
+    geos::geom::CoordinateSequence sequence;
     sequence.add(coord);
 
-    PointPtr pt = factory_->createPoint(sequence);
+    auto pt = factory_->createPoint(sequence);
 
     ensure("createPoint() returned null pointer.", pt != nullptr);
     ensure("createPoint() returned empty point.", !pt->isEmpty());
@@ -385,11 +361,11 @@ void object::test<11>
     ensure(pt->isValid());
     ensure(pt->getCoordinate() != nullptr);
 
-    CoordinateCPtr pcoord = pt->getCoordinate();
-    ensure(pcoord != nullptr);
-    ensure_equals(pcoord->x, x_);
-    ensure_equals(pcoord->y, y_);
-    ensure_equals(pcoord->z, z_);
+    Coordinate pcoord;
+    pt->getCoordinatesRO()->getAt(0, pcoord);
+    ensure_equals(pcoord.x, x_);
+    ensure_equals(pcoord.y, y_);
+    ensure_equals(pcoord.z, z_);
 
     std::unique_ptr<geos::geom::Geometry> geo;
     geo = pt->getEnvelope();
@@ -414,9 +390,6 @@ void object::test<11>
     ensure_equals(pt->getNumPoints(), 1u);
     ensure_equals(pt->getLength(), 0.0);
     ensure_equals(pt->getArea(), 0.0);
-
-    // FREE MEMORY
-    factory_->destroyGeometry(pt);
 }
 
 // Test of createLinearRing() const
@@ -444,24 +417,25 @@ void object::test<12>
 
     ensure_equals(lr->getGeometryTypeId(), geos::geom::GEOS_LINEARRING);
     ensure_equals(lr->getDimension(), geos::geom::Dimension::L);
+    ensure_equals(lr->getCoordinateDimension(), 2u);
     ensure_equals(lr->getBoundaryDimension(), geos::geom::Dimension::False);
     ensure_equals(lr->getNumPoints(), 0u);
     ensure_equals(lr->getLength(), 0.0);
     ensure_equals(lr->getArea(), 0.0);
 }
 
-// Test of createLinearRing(CoordinateSequence* newCoords) const
+// Test of createLinearRing(std::unique_ptr<CoordinateSequence>&& newCoords) const
 template<>
 template<>
 void object::test<13>
 ()
 {
     const std::size_t size = 5;
-    CoordArrayPtr coords = new geos::geom::CoordinateArraySequence(size);
+    auto coords = geos::detail::make_unique<CoordinateSequence>(size);
     ensure(coords != nullptr);
     ensure_equals(coords->getSize(), size);
 
-    LinearRingPtr lr = factory_->createLinearRing(coords);
+    auto lr = factory_->createLinearRing(std::move(coords));
     ensure("createLinearRing() returned null pointer.", lr != nullptr);
     ensure("createLinearRing() returned empty point.", !lr->isEmpty());
     ensure(lr->isSimple());
@@ -476,9 +450,6 @@ void object::test<13>
     ensure_equals(lr->getNumPoints(), size);
     ensure_equals(lr->getLength(), 0.0);
     ensure_equals(lr->getArea(), 0.0);
-
-    // FREE MEMORY
-    factory_->destroyGeometry(lr);
 }
 
 // Test of createLinearRing(const CoordinateSequence& coordinates) const
@@ -488,10 +459,10 @@ void object::test<14>
 ()
 {
     const std::size_t size = 5;
-    geos::geom::CoordinateArraySequence coords(size);
+    geos::geom::CoordinateSequence coords(size);
     ensure_equals(coords.getSize(), size);
 
-    LinearRingPtr lr = factory_->createLinearRing(coords);
+    auto lr = factory_->createLinearRing(coords);
     ensure("createLinearRing() returned empty point.", !lr->isEmpty());
     ensure_equals(lr->getNumPoints(), size);
     ensure(lr->isSimple());
@@ -503,9 +474,6 @@ void object::test<14>
     ensure_equals(lr->getNumPoints(), size);
     ensure_equals(lr->getLength(), 0.0);
     ensure_equals(lr->getArea(), 0.0);
-
-    // FREE MEMORY
-    factory_->destroyGeometry(lr);
 }
 
 // Test of createLineString() const
@@ -541,24 +509,25 @@ void object::test<15>
 
     ensure_equals(line->getGeometryTypeId(), geos::geom::GEOS_LINESTRING);
     ensure_equals(line->getDimension(), geos::geom::Dimension::L);
+    ensure_equals(line->getCoordinateDimension(), 2u);
     ensure_equals(line->getBoundaryDimension(), geos::geom::Dimension::P);   // empty linestring
     ensure_equals(line->getNumPoints(), 0u);
     ensure_equals(line->getLength(), 0.0);
     ensure_equals(line->getArea(), 0.0);
 }
 
-// Test of createLineString(CoordinateSequence* coordinates) const
+// Test of createLineString(std::unique_ptr<CoordinateSequence>&& coordinates) const
 template<>
 template<>
 void object::test<16>
 ()
 {
     const std::size_t size = 5;
-    CoordArrayPtr coords = new geos::geom::CoordinateArraySequence(size);
+    auto coords = geos::detail::make_unique<CoordinateSequence>(size);
     ensure(coords != nullptr);
     ensure_equals(coords->getSize(), size);
 
-    LineStringPtr line = factory_->createLineString(coords);
+    auto line = factory_->createLineString(std::move(coords));
     ensure("createLineString() returned null pointer.", line != nullptr);
     ensure("createLineString() returned empty point.", !line->isEmpty());
     ensure(line->isSimple());
@@ -573,9 +542,6 @@ void object::test<16>
     ensure_equals(line->getNumPoints(), size);
     ensure_equals(line->getLength(), 0.0);
     ensure_equals(line->getArea(), 0.0);
-
-    // FREE MEMORY
-    factory_->destroyGeometry(line);
 }
 
 // Test of createLineString(const CoordinateSequence& coordinates) const
@@ -585,10 +551,10 @@ void object::test<17>
 ()
 {
     const std::size_t size = 5;
-    geos::geom::CoordinateArraySequence coords(size);
+    geos::geom::CoordinateSequence coords(size);
     ensure_equals(coords.getSize(), size);
 
-    LineStringPtr line = factory_->createLineString(coords);
+    auto line = factory_->createLineString(coords);
     ensure("createLineString() returned empty point.", !line->isEmpty());
     ensure_equals(line->getNumPoints(), size);
     ensure(line->isSimple());
@@ -600,10 +566,8 @@ void object::test<17>
     ensure_equals(line->getNumPoints(), size);
     ensure_equals(line->getLength(), 0.0);
     ensure_equals(line->getArea(), 0.0);
-
-    // FREE MEMORY
-    factory_->destroyGeometry(line);
 }
+
 // Test of createPolygon() const
 template<>
 template<>
@@ -657,7 +621,7 @@ void object::test<19>
     const std::size_t size = 7;
 
     // Create sequence of coordinates
-    CoordArrayPtr coords = new geos::geom::CoordinateArraySequence(size);
+    auto coords = geos::detail::make_unique<CoordinateSequence>(size);
     ensure(coords != nullptr);
     coords->setAt(Coordinate(0, 10), 0);
     coords->setAt(Coordinate(5, 5), 1);
@@ -669,7 +633,7 @@ void object::test<19>
     ensure_equals(coords->getSize(), size);
 
     // Create exterior ring
-    LinearRingPtr exterior = factory_->createLinearRing(coords);
+    auto exterior = factory_->createLinearRing(std::move(coords));
     ensure("createLinearRing returned null pointer.", exterior != nullptr);
     ensure("createLinearRing() returned empty point.", !exterior->isEmpty());
     ensure(exterior->isSimple());
@@ -681,7 +645,7 @@ void object::test<19>
     ensure(exterior->getLength() != 0.0);
 
     // Create polygon
-    PolygonPtr poly = factory_->createPolygon(exterior, nullptr);
+    auto poly = factory_->createPolygon(std::move(exterior));
     ensure("createPolygon returned null pointer.", poly != nullptr);
     ensure("createPolygon() returned empty point.", !poly->isEmpty());
     ensure(poly->isSimple());
@@ -691,9 +655,6 @@ void object::test<19>
     ensure_equals(poly->getNumPoints(), size);
     ensure(poly->getArea() != 0.0);
     ensure(poly->getLength() != 0.0);
-
-    // FREE MEMORY
-    factory_->destroyGeometry(poly);
 }
 
 // Test of createPolygon(const LinearRing& shell, const std::vector<Geometry*>& holes) const
@@ -707,7 +668,7 @@ void object::test<20>
     const std::size_t interiorSize = 5;
 
     // Create sequence of coordinates
-    CoordArrayPtr coords = new geos::geom::CoordinateArraySequence(exteriorSize);
+    auto coords = geos::detail::make_unique<CoordinateSequence>(exteriorSize);
     ensure(coords != nullptr);
     coords->setAt(Coordinate(0, 10), 0);
     coords->setAt(Coordinate(5, 5), 1);
@@ -719,7 +680,7 @@ void object::test<20>
     ensure_equals(coords->getSize(), exteriorSize);
 
     // Create exterior ring
-    LinearRingPtr exterior = factory_->createLinearRing(coords);
+    auto exterior = factory_->createLinearRing(std::move(coords));
     ensure("createLinearRing returned null pointer.", exterior != nullptr);
     ensure("createLinearRing() returned empty point.", !exterior->isEmpty());
     ensure(exterior->isSimple());
@@ -731,20 +692,16 @@ void object::test<20>
     ensure(exterior->getLength() != 0.0);
 
     // Create collection of holes
-    auto geo = reader_.read(("LINEARRING(7 7, 12 7, 12 12, 7 12, 7 7)"));
-    ensure(geo != nullptr);
-
-    LinearRingPtr hole = dynamic_cast<LinearRingPtr>(geo.release());
+    auto hole = reader_.read<LinearRing>(("LINEARRING(7 7, 12 7, 12 12, 7 12, 7 7)"));
     ensure(hole != nullptr);
     ensure(hole->isRing());
     ensure_equals(hole->getNumPoints(), interiorSize);
 
-    // REMEMBER TO DEALLOCATE THIS COLLECTION
-    std::vector<LinearRingPtr> holes;
-    holes.push_back(hole);
+    std::vector<decltype(hole)> holes;
+    holes.emplace_back(std::move(hole));
 
     // Create polygon using copy ctor
-    PolygonPtr poly = factory_->createPolygon((*exterior), holes);
+    auto poly = factory_->createPolygon(std::move(exterior), std::move(holes));
     ensure("createPolygon returned null pointer.", poly != nullptr);
     ensure("createPolygon() returned empty point.", !poly->isEmpty());
     ensure(poly->isSimple());
@@ -757,15 +714,6 @@ void object::test<20>
 
     ensure_equals(poly->getNumGeometries(), 1u);
     ensure_equals(poly->getNumInteriorRing(), 1u);
-
-    // FREE MEMORY
-    for(auto& h : holes) {
-        delete h;
-    }
-    holes.clear();
-
-    factory_->destroyGeometry(exterior);
-    factory_->destroyGeometry(poly);
 }
 
 // Test of createGeometryCollection() const
@@ -781,6 +729,7 @@ void object::test<21>
     ensure(col->isValid());
     ensure(col->getCentroid()->isEmpty());
     ensure_equals(col->getGeometryTypeId(), geos::geom::GEOS_GEOMETRYCOLLECTION);
+    ensure_equals(col->getCoordinateDimension(), 2u);
     ensure_equals(col->getDimension(), geos::geom::Dimension::False);
     ensure_equals(col->getBoundaryDimension(), geos::geom::Dimension::False);
     ensure_equals(col->getNumPoints(), 0u);
@@ -797,35 +746,32 @@ void object::test<22>
     using geos::geom::Coordinate;
 
     // Buffer for geometries
-    std::vector<GeometryPtr>* vec = new std::vector<GeometryPtr>();
+    std::vector<std::unique_ptr<Geometry>> vec;
 
     // Add single point
     Coordinate coord(x_, y_, z_);
-    GeometryPtr point = factory_->createPoint(coord);
+    std::unique_ptr<Geometry> point(factory_->createPoint(coord));
     ensure(point != nullptr);
-    vec->push_back(point);
+    vec.push_back(std::move(point));
 
     // Add single LineString
-    CoordArrayPtr coords = new geos::geom::CoordinateArraySequence(3);
+    auto coords = geos::detail::make_unique<CoordinateSequence>(3u);
     ensure(coords != nullptr);
     coords->setAt(Coordinate(0, 0), 0);
     coords->setAt(Coordinate(5, 5), 1);
     coords->setAt(Coordinate(10, 5), 2);
     ensure_equals(coords->getSize(), 3u);
-    GeometryPtr line = factory_->createLineString(coords);
-    vec->push_back(line);
+    auto line = factory_->createLineString(std::move(coords));
+    vec.push_back(std::move(line));
 
     // Create geometry collection
-    GeometryColPtr col = factory_->createGeometryCollection(vec);
-    ensure(coords != nullptr);
+    auto col = factory_->createGeometryCollection(std::move(vec));
+    ensure(col != nullptr);
     ensure_equals(col->getGeometryTypeId(), geos::geom::GEOS_GEOMETRYCOLLECTION);
     ensure_equals(col->getNumGeometries(), 2u);
-
-    // FREE MEMORY
-    factory_->destroyGeometry(col);
 }
 
-// Test of createGeometryCollection(const std::vector<Geometry*>& newGeoms) const
+// Test of createGeometryCollection(const std::vector<const Geometry*>& newGeoms) const
 template<>
 template<>
 void object::test<23>
@@ -836,33 +782,26 @@ void object::test<23>
 
     std::vector<const geos::geom::Geometry*> vec;
 
-    GeometryPtr geo = nullptr;
-    geo = factory_->createPoint(coord);
-    vec.push_back(geo);
+    auto geo1 = factory_->createPoint(coord);
+    vec.push_back(geo1.get());
 
     coord.x *= 2;
     coord.y *= 2;
     coord.z *= 2;
-    geo = factory_->createPoint(coord);
-    vec.push_back(geo);
+    auto geo2 = factory_->createPoint(coord);
+    vec.push_back(geo2.get());
 
     coord.x *= 3;
     coord.y *= 3;
     coord.z *= 3;
-    geo = factory_->createPoint(coord);
-    vec.push_back(geo);
+    auto geo3 = factory_->createPoint(coord);
+    vec.push_back(geo3.get());
 
     // Factory creates copy of the vec collection
-    GeometryColPtr col = factory_->createGeometryCollection(vec);
+    auto col = factory_->createGeometryCollection(vec);
     ensure(col != nullptr);
     ensure_equals(col->getGeometryTypeId(), geos::geom::GEOS_GEOMETRYCOLLECTION);
     ensure_equals(col->getNumGeometries(), size);
-
-    // FREE MEMORY
-    factory_->destroyGeometry(col);
-    for(auto& g : vec) {
-        delete g;
-    }
 }
 
 // Test of createMultiPoint() const
@@ -897,6 +836,7 @@ void object::test<24>
     ensure(geo->isEmpty());
 
     ensure_equals(mp->getGeometryTypeId(), geos::geom::GEOS_MULTIPOINT);
+    ensure_equals(mp->getCoordinateDimension(), 2u);
     ensure_equals(mp->getDimension(), geos::geom::Dimension::P);
     ensure_equals(mp->getBoundaryDimension(), geos::geom::Dimension::False);
     ensure_equals(mp->getNumPoints(), 0u);
@@ -904,7 +844,7 @@ void object::test<24>
     ensure_equals(mp->getArea(), 0.0);
 }
 
-// Test of createMultiPoint(std::vector<Geometry*>* newPoints) const
+// Test of createMultiPoint(std::vector<std::unique_ptr<Geometry>>&& newPoints) const
 template<>
 template<>
 void object::test<25>
@@ -913,40 +853,36 @@ void object::test<25>
     const std::size_t size = 3;
     geos::geom::Coordinate coord(x_, y_, z_);
 
-    std::vector<GeometryPtr>* vec = new std::vector<GeometryPtr>();
+    std::vector<std::unique_ptr<Geometry>> vec;
 
-    GeometryPtr geo = nullptr;
-    geo = factory_->createPoint(coord);
+    auto geo = factory_->createPoint(coord);
     ensure(geo != nullptr);
-    vec->push_back(geo);
+    vec.push_back(std::move(geo));
 
     coord.x *= 2;
     coord.y *= 2;
     coord.z *= 2;
     geo = factory_->createPoint(coord);
     ensure(geo != nullptr);
-    vec->push_back(geo);
+    vec.push_back(std::move(geo));
 
     coord.x *= 3;
     coord.y *= 3;
     coord.z *= 3;
     geo = factory_->createPoint(coord);
     ensure(geo != nullptr);
-    vec->push_back(geo);
+    vec.push_back(std::move(geo));
 
     // Factory creates copy of the vec collection
-    MultiPointPtr mp = factory_->createMultiPoint(vec);
+    auto mp = factory_->createMultiPoint(std::move(vec));
     ensure(mp != nullptr);
     ensure(mp->isValid());
     ensure(mp->isSimple());
     ensure_equals(mp->getNumGeometries(), size);
     ensure_equals(mp->getGeometryTypeId(), geos::geom::GEOS_MULTIPOINT);
-
-    // FREE MEMORY
-    factory_->destroyGeometry(mp);
 }
 
-// Test of createMultiPoint(const std::vector<Geometry*>& fromPoints) const
+// Test of createMultiPoint(const std::vector<const Geometry*>& fromPoints) const
 template<>
 template<>
 void object::test<26>
@@ -957,35 +893,28 @@ void object::test<26>
 
     std::vector<const geos::geom::Geometry*> vec;
 
-    GeometryPtr geo = nullptr;
-    geo = factory_->createPoint(coord);
-    vec.push_back(geo);
+    auto geo1 = factory_->createPoint(coord);
+    vec.push_back(geo1.get());
 
     coord.x *= 2;
     coord.y *= 2;
     coord.z *= 2;
-    geo = factory_->createPoint(coord);
-    vec.push_back(geo);
+    auto geo2 = factory_->createPoint(coord);
+    vec.push_back(geo2.get());
 
     coord.x *= 3;
     coord.y *= 3;
     coord.z *= 3;
-    geo = factory_->createPoint(coord);
-    vec.push_back(geo);
+    auto geo3 = factory_->createPoint(coord);
+    vec.push_back(geo3.get());
 
     // Factory creates copy of the vec collection
-    MultiPointPtr mp = factory_->createMultiPoint(vec);
+    auto mp = factory_->createMultiPoint(vec);
     ensure(mp != nullptr);
     ensure(mp->isValid());
     ensure(mp->isSimple());
     ensure_equals(mp->getNumGeometries(), size);
     ensure_equals(mp->getGeometryTypeId(), geos::geom::GEOS_MULTIPOINT);
-
-    // FREE MEMORY
-    factory_->destroyGeometry(mp);
-    for(auto& g : vec) {
-        delete g;
-    }
 }
 
 // Test of createMultiPoint(const CoordinateSequence& fromCoords) const
@@ -998,21 +927,18 @@ void object::test<27>
     const std::size_t size = 3;
 
     // Add collection of coordinates
-    geos::geom::CoordinateArraySequence coords(size);
+    geos::geom::CoordinateSequence coords(size);
     coords.setAt(Coordinate(0, 0), 0);
     coords.setAt(Coordinate(5, 5), 1);
     coords.setAt(Coordinate(10, 5), 2);
     ensure_equals(coords.getSize(), size);
 
-    MultiPointPtr mp = factory_->createMultiPoint(coords);
+    auto mp = factory_->createMultiPoint(coords);
     ensure(mp != nullptr);
     ensure(mp->isValid());
     ensure(mp->isSimple());
     ensure_equals(mp->getNumGeometries(), size);
     ensure_equals(mp->getGeometryTypeId(), geos::geom::GEOS_MULTIPOINT);
-
-    // FREE MEMORY
-    factory_->destroyGeometry(mp);
 }
 
 // Test of createMultiLineString() const
@@ -1047,6 +973,7 @@ void object::test<28>
     ensure(geo->isEmpty());
 
     ensure_equals(mls->getGeometryTypeId(), geos::geom::GEOS_MULTILINESTRING);
+    ensure_equals(mls->getCoordinateDimension(), 2u);
     ensure_equals(mls->getDimension(), geos::geom::Dimension::L);
     ensure_equals(mls->getBoundaryDimension(), geos::geom::Dimension::P);
     ensure_equals(mls->getNumPoints(), 0u);
@@ -1054,7 +981,7 @@ void object::test<28>
     ensure_equals(mls->getArea(), 0.0);
 }
 
-// Test of createMultiLineString(std::vector<Geometry*>* newLines) const
+// Test of createMultiLineString(std::vector<std::unique_ptr<Geometry>>&& newLines) const
 template<>
 template<>
 void object::test<29>
@@ -1065,35 +992,32 @@ void object::test<29>
     const std::size_t size = 5;
     const std::size_t lineSize = 2;
 
-    std::vector<GeometryPtr>* lines = new std::vector<GeometryPtr>();
+    std::vector<std::unique_ptr<Geometry>> lines;
 
     for(std::size_t i = 0; i < size; ++i) {
         const double factor = static_cast<double>(i * i);
-        CoordArrayPtr coords = new geos::geom::CoordinateArraySequence(lineSize);
+        auto coords = geos::detail::make_unique<geos::geom::CoordinateSequence>(lineSize);
         ensure(coords != nullptr);
         coords->setAt(Coordinate(0. + factor, 0. + factor), 0);
         coords->setAt(Coordinate(5. + factor, 5. + factor), 1);
         ensure_equals(coords->getSize(), lineSize);
 
-        LineStringPtr line = factory_->createLineString(coords);
+        auto line = factory_->createLineString(std::move(coords));
         ensure("createLineString() returned empty point.", !line->isEmpty());
         ensure_equals(line->getNumPoints(), lineSize);
         ensure(line->isSimple());
         ensure(line->getCoordinate() != nullptr);
         ensure_equals(line->getGeometryTypeId(), geos::geom::GEOS_LINESTRING);
 
-        lines->push_back(line);
+        lines.push_back(std::move(line));
     }
 
-    MultiLineStringPtr mls = factory_->createMultiLineString(lines);
+    auto mls = factory_->createMultiLineString(std::move(lines));
     ensure(mls != nullptr);
     // TODO - mloskot - why isValid() returns false?
     //ensure( mls->isValid() );
     ensure_equals(mls->getNumGeometries(), size);
     ensure_equals(mls->getGeometryTypeId(), geos::geom::GEOS_MULTILINESTRING);
-
-    // FREE MEMORY
-    factory_->destroyGeometry(mls);
 }
 
 // Test of createMultiLineString(const std::vector<Geometry*>& fromLines) const
@@ -1107,38 +1031,32 @@ void object::test<30>
     const std::size_t size = 5;
     const std::size_t lineSize = 2;
 
-    std::vector<const geos::geom::Geometry*> lines;
+    std::vector<std::unique_ptr<geos::geom::Geometry>> lines;
 
     for(std::size_t i = 0; i < size; ++i) {
         const double factor = static_cast<double>(i * i);
-        CoordArrayPtr coords = new geos::geom::CoordinateArraySequence(lineSize);
+        auto coords = geos::detail::make_unique<CoordinateSequence>(lineSize);
         ensure(coords != nullptr);
         coords->setAt(Coordinate(0. + factor, 0. + factor), 0);
         coords->setAt(Coordinate(5. + factor, 5. + factor), 1);
         ensure_equals(coords->getSize(), lineSize);
 
-        LineStringPtr line = factory_->createLineString(coords);
+        auto line = factory_->createLineString(std::move(coords));
         ensure("createLineString() returned empty point.", !line->isEmpty());
         ensure_equals(line->getNumPoints(), lineSize);
         ensure(line->isSimple());
         ensure(line->getCoordinate() != nullptr);
         ensure_equals(line->getGeometryTypeId(), geos::geom::GEOS_LINESTRING);
 
-        lines.push_back(line);
+        lines.push_back(std::move(line));
     }
 
-    MultiLineStringPtr mls = factory_->createMultiLineString(lines);
+    auto mls = factory_->createMultiLineString(std::move(lines));
     ensure(mls != nullptr);
     // TODO - mloskot - why isValid() returns false?
     //ensure( mls->isValid() );
     ensure_equals(mls->getNumGeometries(), size);
     ensure_equals(mls->getGeometryTypeId(), geos::geom::GEOS_MULTILINESTRING);
-
-    // FREE MEMORY
-    factory_->destroyGeometry(mls);
-    for(auto& g : lines) {
-        delete g;
-    }
 }
 
 // Test of createMultiPolygon() const
@@ -1147,8 +1065,11 @@ template<>
 void object::test<31>
 ()
 {
-    // TODO - mloskot
-    //inform("Test not implemented!");
+    auto mp = factory_->createMultiPolygon();
+
+    ensure_equals(mp->getGeometryTypeId(), geos::geom::GEOS_MULTIPOLYGON);
+    ensure_equals(mp->getCoordinateDimension(), 2u);
+    ensure_equals(mp->getDimension(), geos::geom::Dimension::A);
 }
 
 // Test of createMultiPolygon(std::vector<Geometry>* newPolys) const
@@ -1198,7 +1119,6 @@ template<>
 void object::test<36>
 ()
 {
-    typedef std::unique_ptr<geos::geom::Geometry> GeometryAutoPtr;
     typedef std::vector<PointPtr> PointVect;
 
     const std::size_t size = 3;
@@ -1206,33 +1126,26 @@ void object::test<36>
 
     PointVect vec;
 
-    PointPtr geo = nullptr;
-    geo = factory_->createPoint(coord);
-    vec.push_back(geo);
+    auto geo1 = factory_->createPoint(coord);
+    vec.push_back(geo1.get());
 
     coord.x *= 2;
     coord.y *= 2;
     coord.z *= 2;
-    geo = factory_->createPoint(coord);
-    vec.push_back(geo);
+    auto geo2 = factory_->createPoint(coord);
+    vec.push_back(geo2.get());
 
     coord.x *= 3;
     coord.y *= 3;
     coord.z *= 3;
-    geo = factory_->createPoint(coord);
-    vec.push_back(geo);
+    auto geo3 = factory_->createPoint(coord);
+    vec.push_back(geo3.get());
 
     // Factory creates copy of the vec collection
-    GeometryAutoPtr g = factory_->buildGeometry(vec.begin(), vec.end());
+    auto g = factory_->buildGeometry(vec.begin(), vec.end());
     ensure(g.get() != nullptr);
     ensure_equals(g->getGeometryTypeId(), geos::geom::GEOS_MULTIPOINT);
     ensure_equals(g->getNumGeometries(), size);
-
-    // FREE MEMORY
-    PointVect::const_iterator it;
-    for(it = vec.begin(); it != vec.end(); ++it) {
-        delete(*it);
-    }
 }
 
 // Test of
@@ -1252,4 +1165,156 @@ void object::test<37>
     ensure_equals(mp->getNumGeometries(), 2u);
 }
 
+// Test of createPoint(const Coordinate &coordinate) const
+template<>
+template<>
+void object::test<38>
+()
+{
+    geos::geom::CoordinateXY coord(x_, y_);
+
+    auto pt = factory_->createPoint(coord);
+
+    ensure("createPoint() returned null pointer.", pt != nullptr);
+    ensure("createPoint() returned empty point.", !pt->isEmpty());
+    ensure(pt->isSimple());
+    ensure(pt->isValid());
+    ensure(pt->getCoordinate() != nullptr);
+
+    const CoordinateXY* pcoord = pt->getCoordinate();
+    ensure(pcoord != nullptr);
+    ensure_equals(pcoord->x, x_);
+    ensure_equals(pcoord->y, y_);
+
+    std::unique_ptr<geos::geom::Geometry> geo;
+    geo = pt->getEnvelope();
+    ensure(geo != nullptr);
+    ensure(!geo->isEmpty());
+
+    geo = pt->getCentroid();
+    ensure(geo != nullptr);
+    ensure(!geo->isEmpty());
+
+    geo = pt->getBoundary();
+    ensure(geo != nullptr);
+    ensure(geo->isEmpty());
+
+    geo = pt->convexHull();
+    ensure(geo != nullptr);
+    ensure(!geo->isEmpty());
+
+    ensure_equals(pt->getCoordinateDimension(), 2u);
+    ensure_equals(pt->getGeometryTypeId(), geos::geom::GEOS_POINT);
+    ensure_equals(pt->getDimension(), geos::geom::Dimension::P);
+    ensure_equals(pt->getBoundaryDimension(), geos::geom::Dimension::False);
+    ensure_equals(pt->getNumPoints(), 1u);
+    ensure_equals(pt->getLength(), 0.0);
+    ensure_equals(pt->getArea(), 0.0);
+}
+
+// Test of createPoint for various coordinate types
+template<>
+template<>
+void object::test<39>
+()
+{
+    // XY
+    auto pt_xy = factory_->createPoint(CoordinateXY{1, 2});
+    CoordinateXY xy_out;
+    ensure_equals(pt_xy->getCoordinateDimension(), 2u);
+    pt_xy->getCoordinatesRO()->getAt(0, xy_out);
+    ensure_equals(xy_out, CoordinateXY{1, 2});
+    ensure(!pt_xy->hasZ());
+    ensure(!pt_xy->hasM());
+
+    // XYZ
+    auto pt_xyz = factory_->createPoint(Coordinate{1, 2, 3});
+    Coordinate xyz_out;
+    pt_xyz->getCoordinatesRO()->getAt(0, xyz_out);
+    ensure_equals(pt_xyz->getCoordinateDimension(), 3u);
+    ensure_equals_xyz(xyz_out, Coordinate{1, 2, 3});
+    ensure(pt_xyz->hasZ());
+    ensure(!pt_xyz->hasM());
+
+    // XYM
+    auto pt_xym = factory_->createPoint(CoordinateXYM{1, 2, 3});
+    CoordinateXYM xym_out;
+    pt_xym->getCoordinatesRO()->getAt(0, xym_out);
+    ensure_equals(pt_xym->getCoordinateDimension(), 3u);
+    ensure_equals_xym(xym_out, CoordinateXYM{1, 2, 3});
+    ensure(!pt_xym->hasZ());
+    ensure(pt_xym->hasM());
+
+    // XYZM
+    auto pt_xyzm = factory_->createPoint(CoordinateXYZM{1, 2, 3, 4});
+    CoordinateXYZM xyzm_out;
+    pt_xyzm->getCoordinatesRO()->getAt(0, xyzm_out);
+    ensure_equals(pt_xyzm->getCoordinateDimension(), 4u);
+    ensure_equals_xyzm(xyzm_out, CoordinateXYZM{1, 2, 3, 4});
+    ensure(pt_xyzm->hasZ());
+    ensure(pt_xyzm->hasM());
+}
+
+// createMultiPoint(CoordinateSequence) preserves dimension
+template<>
+template<>
+void object::test<40>
+()
+{
+    // XY
+    CoordinateSequence xy_seq{CoordinateXY{1,2}, CoordinateXY{3,4}};
+    auto mp_xy = factory_->createMultiPoint(xy_seq);
+    ensure_equals(mp_xy->getCoordinateDimension(), 2u);
+    ensure_equals(mp_xy->getNumGeometries(), 2u);
+    ensure(!mp_xy->hasZ());
+    ensure(!mp_xy->hasM());
+
+    // XYZ
+    CoordinateSequence xyz_seq{Coordinate{1,2,3}, Coordinate{4,5,6}};
+    auto mp_xyz = factory_->createMultiPoint(xyz_seq);
+    ensure_equals(mp_xyz->getCoordinateDimension(), 3u);
+    ensure_equals(mp_xy->getNumGeometries(), 2u);
+    ensure(mp_xyz->hasZ());
+    ensure(!mp_xyz->hasM());
+
+    // XYM
+    CoordinateSequence xym_seq{CoordinateXYM{1,2,3}, CoordinateXYM{4,5,6}};
+    auto mp_xym = factory_->createMultiPoint(xym_seq);
+    ensure_equals(mp_xym->getCoordinateDimension(), 3u);
+    ensure_equals(mp_xy->getNumGeometries(), 2u);
+    ensure(!mp_xym->hasZ());
+    ensure(mp_xym->hasM());
+
+    // XYZM
+    CoordinateSequence xyzm_seq{CoordinateXYZM{1,2,3,4}, CoordinateXYZM{5,6,7,8}};
+    auto mp_xyzm = factory_->createMultiPoint(xyzm_seq);
+    ensure_equals(mp_xyzm->getCoordinateDimension(), 4u);
+    ensure_equals(mp_xy->getNumGeometries(), 2u);
+    ensure(mp_xyzm->hasZ());
+    ensure(mp_xyzm->hasM());
+}
+
+
+// Test createEmpty and createMulti
+template<>
+template<>
+void object::test<41>
+()
+{
+    auto g1 = factory_->createEmpty(geos::geom::GEOS_MULTIPOINT);
+    auto g2 = factory_->createEmpty(geos::geom::GEOS_POINT);
+    auto mg1 = factory_->createMulti(std::move(g1));
+    auto mg2 = factory_->createMulti(std::move(g2));
+    ensure(mg1->isEmpty());
+    ensure(mg2->isEmpty());
+    g1 = reader_.read("POINT(1 1)");
+    g2 = reader_.read("MULTIPOINT((1 1))");
+    mg1 = factory_->createMulti(std::move(g1));
+    mg2 = factory_->createMulti(std::move(g2));
+    g2 = reader_.read("MULTIPOINT((1 1))");
+    ensure_equals_geometry(g2.get(), mg1.get());
+    ensure_equals_geometry(g2.get(), mg2.get());
+}
+
+
 } // namespace tut
